package com.bluesky.skyline.workflow.spikes;

import java.util.Collection;
import java.util.Iterator;
import org.jbpm.api.Execution;
import org.jbpm.api.ProcessInstance;
import org.jbpm.pvm.internal.model.ExecutionImpl;
import org.jbpm.test.JbpmTestCase;
import org.hibernate.SessionFactory;




public class ReworkPublicTest extends JbpmTestCase{
	
	//We flow throuh the work flow just to show it works
	public void testNormal() {
		
		 deployTestProcess();
		 ProcessInstance processInstance = executionService.startProcessInstanceByKey("ForkAssumptionTest");
		 processInstance = executionService.signalExecutionById(processInstance.getId());
		 Execution node = processInstance.findActiveExecutionIn("state2");
		 Execution bottomPath = processInstance.findActiveExecutionIn("state3");
		 processInstance = executionService.signalExecutionById(node.getId(),"to join1");
		 processInstance = executionService.signalExecutionById(bottomPath.getId(),"to join1");
		 assertNotNull(processInstance.findActiveExecutionIn("state4"));
		
	}
	
	public void testReworkAssumptions() {
		
		 deployTestProcess();
		 ProcessInstance processInstance = executionService.startProcessInstanceByKey("ForkAssumptionTest");
		 processInstance = executionService.signalExecutionById(processInstance.getId());
		 Execution node = processInstance.findActiveExecutionIn("state2");
		 executionService.setVariable(node.getId(), "SKYLINE_REWORK_TARGET", "to state1");
		 node = executionService.signalExecutionById(node.getId(),"to rework");
		 //Lets' end the bottom execution
		 ExecutionImpl bottomPath = (ExecutionImpl) processInstance.findActiveExecutionIn("state3");
		 bottomPath.end();
		 assertTrue(bottomPath.isEnded());  

		 ExecutionImpl p=  (ExecutionImpl)executionService.findExecutionById(processInstance.getId()); 
		 assertNull(p.getActivityName()); //In 4.1 this was "state1"
		 Collection<ExecutionImpl> childExecutions = p.getExecutions();
		 assertEquals(2,childExecutions.size()); //In 4.1 this was 1 the only item on the colllection was "state1"
		 Iterator<ExecutionImpl> i = childExecutions.iterator();
		 ExecutionImpl o =i.next();
		 assertEquals("state1",o.getActivityName());
		 ExecutionImpl o2 = i.next();
		 assertEquals("state3",o2.getActivityName());
		
		 assertEquals(bottomPath.getId(),o2.getId());   //They are the same process
		 assertFalse(o2.isEnded());  //But now it is not ended!
		executionService.signalExecutionById(o.getId(),"to fork1");
		
	}
	
	  /**
	   * Returns deployment db id
	   */
	  private String deployTestProcess() {
	    return deployJpdlXmlString(
	    		"<process name='ForkAssumptionTest' xmlns='http://jbpm.org/4.0/jpdl'>"+
	    		   "<start name='start1'>"+
	    		      "<transition name='to state1' to='state1'/>"+
	    		   "</start>"+
	    		   "<state name='state1'>"+
	    		      "<transition name='to fork1' to='fork1'/>"+
	    		   "</state>"+
	    		   "<state name='state2'>"+
	    		      "<transition name='to rework' to='rework'/>"+
	    		      "<transition name='to join1' to='join1'/>"+
	    		   "</state>"+
	    		   "<state name='state3'>"+
	    		      "<transition name='to join1' to='join1'/>"+
	    		      "<transition name='to rework' to='rework'/>"+
	    		   "</state>"+
	    		   "<state name='state4'>"+
	    		      "<transition name='to end1' to='end1'/>"+
	    		      "<transition name='to rework' to='rework'/>"+
	    		   "</state>"+
	    		   "<end name='end1'/>"+
	    		   "<fork name='fork1'>"+
	    		      "<transition name='to state2' to='state2'/>"+
	    		      "<transition name='to state3' to='state3'/>"+
	    		   "</fork>"+
	    		   "<join name='join1'>"+
	    		      "<transition name='to state4' to='state4'/>"+
	    		   "</join>"+
	    		   "<decision expr='#{SKYLINE_REWORK_TARGET}'  name='rework'>"+
	    		      "<transition name='to state1' to='state1'/>"+
	    		      "<transition name='to state3' to='state3'/>"+
	    		      "<transition name='to state2' to='state2'/>"+
	    		   "</decision>"+
	    		"</process>"
	          );
	  }

}

